@@ -1,41 +1,42 @@
 use std::collections::HashMap;
-use std::io::{BufRead, BufReader};
-use std::io::Write;
+use std::io::BufReader;
+use std::ops::DerefMut;
 use std::os::unix::net::UnixStream;
-use std::sync::{Arc, RwLock};
-
-use serde::Serialize;
-use serde::de::DeserializeOwned;
+use std::sync::{Arc, RwLock, RwLockWriteGuard};
+use rand::distributions::Alphanumeric;
+use rand::Rng;
 
 use streamduck_core::core::button::Button;
 use streamduck_core::core::RawButtonPanel;
 use streamduck_core::modules::components::{ComponentDefinition, UIPathValue};
+use streamduck_core::modules::events::SDGlobalEvent;
 use streamduck_core::modules::PluginMetadata;
 use streamduck_core::versions::SOCKET_API;
-use streamduck_core::socket::{parse_packet_to_data, send_no_data_packet_with_requester, send_packet_with_requester, SocketData, SocketPacket};
+use streamduck_core::socket::{send_packet_as_is, SocketPacket};
 use streamduck_daemon::daemon_data::assets::{AddImage, AddImageResult, ListFonts, ListImages, ListImagesResult, RemoveImage, RemoveImageResult};
-use streamduck_daemon::daemon_data::buttons::{AddComponent, AddComponentResult, AddComponentValue, AddComponentValueResult, ClearButton, ClearButtonResult, GetButton, GetButtonResult, GetComponentValues, GetComponentValuesResult, NewButton, NewButtonFromComponent, NewButtonFromComponentResult, NewButtonResult, RemoveComponent, RemoveComponentResult, RemoveComponentValue, RemoveComponentValueResult, SetButton, SetButtonResult, SetComponentValue, SetComponentValueResult};
+use streamduck_daemon::daemon_data::buttons::{AddComponent, AddComponentResult, AddComponentValue, AddComponentValueResult, ClearButton, ClearButtonResult, ClipboardStatusResult, CopyButton, CopyButtonResult, GetButton, GetButtonResult, GetComponentValues, GetComponentValuesResult, NewButton, NewButtonFromComponent, NewButtonFromComponentResult, NewButtonResult, PasteButton, PasteButtonResult, RemoveComponent, RemoveComponentResult, RemoveComponentValue, RemoveComponentValueResult, SetButton, SetButtonResult, SetComponentValue, SetComponentValueResult};
 use streamduck_daemon::daemon_data::config::{ExportDeviceConfig, ExportDeviceConfigResult, GetDeviceConfig, GetDeviceConfigResult, ImportDeviceConfig, ImportDeviceConfigResult, ReloadDeviceConfig, ReloadDeviceConfigResult, ReloadDeviceConfigsResult, SaveDeviceConfig, SaveDeviceConfigResult, SaveDeviceConfigsResult};
 use streamduck_daemon::daemon_data::devices::{AddDevice, AddDeviceResult, Device, GetDevice, GetDeviceResult, ListDevices, RemoveDevice, RemoveDeviceResult, SetBrightness, SetBrightnessResult};
 use streamduck_daemon::daemon_data::modules::{AddModuleValue, AddModuleValueResult, GetModuleValues, GetModuleValuesResult, ListComponents, ListModules, RemoveModuleValue, RemoveModuleValueResult, SetModuleValue, SetModuleValueResult};
 use streamduck_daemon::daemon_data::ops::{CommitChangesToConfig, CommitChangesToConfigResult, DoButtonAction, DoButtonActionResult};
 use streamduck_daemon::daemon_data::panels::{DropStackToRoot, DropStackToRootResult, ForciblyPopScreen, ForciblyPopScreenResult, GetButtonImages, GetButtonImagesResult, GetCurrentScreen, GetCurrentScreenResult, GetStack, GetStackNames, GetStackNamesResult, GetStackResult, PopScreen, PopScreenResult, PushScreen, PushScreenResult, ReplaceScreen, ReplaceScreenResult, ResetStack, ResetStackResult};
 use streamduck_daemon::daemon_data::SocketAPIVersion;
+use streamduck_daemon::UNIX_SOCKET_PATH;
 
-use crate::{SDClient, SDClientError};
+use crate::{SDSyncRequestClient, SDClientError, SDSyncEventClient, SDSyncClient, SDSyncUpcastRequestClient, SDSyncUpcastEventClient};
+use crate::util::{process_request, process_request_without_data, read_response, read_socket};
 
-/// Definition of Unix Socket based client
+/// Unix Socket based Streamduck client
 pub struct UnixClient {
     connection: RwLock<BufReader<UnixStream>>
 }
 
 #[allow(dead_code)]
 impl UnixClient {
-    /// Initializes client using unix domain socket
-    pub fn new() -> Result<Arc<Box<dyn SDClient>>, std::io::Error> {
-        let client: Arc<Box<dyn SDClient>> = Arc::new(Box::new(UnixClient {
-            connection: RwLock::new(BufReader::new(UnixStream::connect("/tmp/streamduck.sock")?))
-        }));
+    fn make_client() -> Result<UnixClient, std::io::Error> {
+        let client = UnixClient {
+            connection: RwLock::new(BufReader::new(UnixStream::connect(UNIX_SOCKET_PATH)?))
+        };
 
         let daemon_version = client.version().expect("Failed to retrieve version");
 
@@ -46,52 +47,41 @@ impl UnixClient {
         Ok(client)
     }
 
-    fn process_request<Req: SocketData + Serialize, Res: SocketData + DeserializeOwned>(&self, request: &Req) -> Result<Res, SDClientError> {
-        let mut handle = self.connection.write().unwrap();
-
-        send_packet_with_requester(handle.get_mut(), "", request)?;
-
-        let mut byte_array = vec![];
-        handle.read_until(0x4, &mut byte_array)?;
-
-        let line = String::from_utf8(byte_array)?;
-
-        let packet: SocketPacket = serde_json::from_str(line.replace("\u{0004}", "").trim())?;
-
-        Ok(parse_packet_to_data(&packet)?)
+    /// Initializes client using unix domain socket
+    pub fn new() -> Result<Arc<dyn SDSyncClient>, std::io::Error> {
+        Ok(Arc::new(UnixClient::make_client()?))
     }
 
-    fn process_request_without_data<Res: SocketData + DeserializeOwned>(&self) -> Result<Res, SDClientError> {
-        let mut handle = self.connection.write().unwrap();
+    /// Initializes client using unix domain socket for use with requests
+    pub fn new_for_requests() -> Result<Arc<dyn SDSyncRequestClient>, std::io::Error> {
+        Ok(Arc::new(UnixClient::make_client()?))
+    }
 
-        send_no_data_packet_with_requester::<Res>(handle.get_mut(), "")?;
+    /// Initializes client using unix domain socket for use with events
+    pub fn new_for_events() -> Result<Arc<dyn SDSyncEventClient>, std::io::Error> {
+        Ok(Arc::new(UnixClient::make_client()?))
+    }
 
-        let mut byte_array = vec![];
-        handle.read_until(0x4, &mut byte_array)?;
-
-        let line = String::from_utf8(byte_array)?;
-
-        let packet: SocketPacket = serde_json::from_str(line.replace("\u{0004}", "").trim())?;
-
-        Ok(parse_packet_to_data(&packet)?)
+    fn get_handle(&self) -> RwLockWriteGuard<BufReader<UnixStream>> {
+        self.connection.write().unwrap()
     }
 }
 
-impl SDClient for UnixClient {
+impl SDSyncRequestClient for UnixClient {
     fn version(&self) -> Result<String, SDClientError> {
-        let response: SocketAPIVersion = self.process_request_without_data()?;
+        let response: SocketAPIVersion = process_request_without_data(self.get_handle().deref_mut())?;
 
         Ok(response.version)
     }
 
     fn device_list(&self) -> Result<Vec<Device>, SDClientError> {
-        let response: ListDevices = self.process_request_without_data()?;
+        let response: ListDevices = process_request_without_data(self.get_handle().deref_mut())?;
 
         Ok(response.devices)
     }
 
     fn get_device(&self, serial_number: &str) -> Result<GetDeviceResult, SDClientError> {
-        let response: GetDeviceResult = self.process_request(&GetDevice {
+        let response: GetDeviceResult = process_request(self.get_handle().deref_mut(), &GetDevice {
             serial_number: serial_number.to_string()
         })?;
 
@@ -99,7 +89,7 @@ impl SDClient for UnixClient {
     }
 
     fn add_device(&self, serial_number: &str) -> Result<AddDeviceResult, SDClientError> {
-        let response: AddDeviceResult = self.process_request(&AddDevice {
+        let response: AddDeviceResult = process_request(self.get_handle().deref_mut(), &AddDevice {
             serial_number: serial_number.to_string()
         })?;
 
@@ -107,7 +97,7 @@ impl SDClient for UnixClient {
     }
 
     fn remove_device(&self, serial_number: &str) -> Result<RemoveDeviceResult, SDClientError> {
-        let response: RemoveDeviceResult = self.process_request(&RemoveDevice {
+        let response: RemoveDeviceResult = process_request(self.get_handle().deref_mut(), &RemoveDevice {
             serial_number: serial_number.to_string()
         })?;
 
@@ -115,13 +105,13 @@ impl SDClient for UnixClient {
     }
 
     fn reload_device_configs(&self) -> Result<ReloadDeviceConfigsResult, SDClientError> {
-        let response: ReloadDeviceConfigsResult = self.process_request_without_data()?;
+        let response: ReloadDeviceConfigsResult = process_request_without_data(self.get_handle().deref_mut())?;
 
         Ok(response)
     }
 
     fn reload_device_config(&self, serial_number: &str) -> Result<ReloadDeviceConfigResult, SDClientError> {
-        let response: ReloadDeviceConfigResult = self.process_request(&ReloadDeviceConfig {
+        let response: ReloadDeviceConfigResult = process_request(self.get_handle().deref_mut(), &ReloadDeviceConfig {
             serial_number: serial_number.to_string()
         })?;
 
@@ -129,13 +119,13 @@ impl SDClient for UnixClient {
     }
 
     fn save_device_configs(&self) -> Result<SaveDeviceConfigsResult, SDClientError> {
-        let response: SaveDeviceConfigsResult = self.process_request_without_data()?;
+        let response: SaveDeviceConfigsResult = process_request_without_data(self.get_handle().deref_mut())?;
 
         Ok(response)
     }
 
     fn save_device_config(&self, serial_number: &str) -> Result<SaveDeviceConfigResult, SDClientError> {
-        let response: SaveDeviceConfigResult = self.process_request(&SaveDeviceConfig {
+        let response: SaveDeviceConfigResult = process_request(self.get_handle().deref_mut(), &SaveDeviceConfig {
             serial_number: serial_number.to_string()
         })?;
 
@@ -143,7 +133,7 @@ impl SDClient for UnixClient {
     }
 
     fn get_device_config(&self, serial_number: &str) -> Result<GetDeviceConfigResult, SDClientError> {
-        let response: GetDeviceConfigResult = self.process_request(&GetDeviceConfig {
+        let response: GetDeviceConfigResult = process_request(self.get_handle().deref_mut(), &GetDeviceConfig {
             serial_number: serial_number.to_string()
         })?;
 
@@ -151,7 +141,7 @@ impl SDClient for UnixClient {
     }
 
     fn import_device_config(&self, serial_number: &str, config: String) -> Result<ImportDeviceConfigResult, SDClientError> {
-        let response: ImportDeviceConfigResult = self.process_request(&ImportDeviceConfig {
+        let response: ImportDeviceConfigResult = process_request(self.get_handle().deref_mut(), &ImportDeviceConfig {
             serial_number: serial_number.to_string(),
             config
         })?;
@@ -160,7 +150,7 @@ impl SDClient for UnixClient {
     }
 
     fn export_device_config(&self, serial_number: &str) -> Result<ExportDeviceConfigResult, SDClientError> {
-        let response: ExportDeviceConfigResult = self.process_request(&ExportDeviceConfig {
+        let response: ExportDeviceConfigResult = process_request(self.get_handle().deref_mut(), &ExportDeviceConfig {
             serial_number: serial_number.to_string()
         })?;
 
@@ -168,7 +158,7 @@ impl SDClient for UnixClient {
     }
 
     fn set_brightness(&self, serial_number: &str, brightness: u8) -> Result<SetBrightnessResult, SDClientError> {
-        let response: SetBrightnessResult = self.process_request(&SetBrightness {
+        let response: SetBrightnessResult = process_request(self.get_handle().deref_mut(), &SetBrightness {
             serial_number: serial_number.to_string(),
             brightness
         })?;
@@ -177,7 +167,7 @@ impl SDClient for UnixClient {
     }
 
     fn list_images(&self, serial_number: &str) -> Result<ListImagesResult, SDClientError> {
-        let response: ListImagesResult = self.process_request(&ListImages {
+        let response: ListImagesResult = process_request(self.get_handle().deref_mut(), &ListImages {
             serial_number: serial_number.to_string()
         })?;
 
@@ -185,7 +175,7 @@ impl SDClient for UnixClient {
     }
 
     fn add_image(&self, serial_number: &str, image_data: &str) -> Result<AddImageResult, SDClientError> {
-        let response: AddImageResult = self.process_request(&AddImage {
+        let response: AddImageResult = process_request(self.get_handle().deref_mut(), &AddImage {
             serial_number: serial_number.to_string(),
             image_data: image_data.to_string()
         })?;
@@ -194,7 +184,7 @@ impl SDClient for UnixClient {
     }
 
     fn remove_image(&self, serial_number: &str, identifier: &str) -> Result<RemoveImageResult, SDClientError> {
-        let response: RemoveImageResult = self.process_request(&RemoveImage {
+        let response: RemoveImageResult = process_request(self.get_handle().deref_mut(), &RemoveImage {
             serial_number: serial_number.to_string(),
             image_identifier: identifier.to_string()
         })?;
@@ -203,25 +193,25 @@ impl SDClient for UnixClient {
     }
 
     fn list_fonts(&self) -> Result<Vec<String>, SDClientError> {
-        let response: ListFonts = self.process_request_without_data()?;
+        let response: ListFonts = process_request_without_data(self.get_handle().deref_mut())?;
 
         Ok(response.font_names)
     }
 
     fn list_modules(&self) -> Result<Vec<PluginMetadata>, SDClientError> {
-        let response: ListModules = self.process_request_without_data()?;
+        let response: ListModules = process_request_without_data(self.get_handle().deref_mut())?;
 
         Ok(response.modules)
     }
 
     fn list_components(&self) -> Result<HashMap<String, HashMap<String, ComponentDefinition>>, SDClientError> {
-        let response: ListComponents = self.process_request_without_data()?;
+        let response: ListComponents = process_request_without_data(self.get_handle().deref_mut())?;
 
         Ok(response.components)
     }
 
     fn get_module_values(&self, module_name: &str) -> Result<GetModuleValuesResult, SDClientError> {
-        let response: GetModuleValuesResult = self.process_request(&GetModuleValues {
+        let response: GetModuleValuesResult = process_request(self.get_handle().deref_mut(), &GetModuleValues {
             module_name: module_name.to_string()
         })?;
 
@@ -229,7 +219,7 @@ impl SDClient for UnixClient {
     }
 
     fn add_module_value(&self, module_name: &str, path: &str) -> Result<AddModuleValueResult, SDClientError> {
-        let response: AddModuleValueResult = self.process_request(&AddModuleValue {
+        let response: AddModuleValueResult = process_request(self.get_handle().deref_mut(), &AddModuleValue {
             module_name: module_name.to_string(),
             path: path.to_string()
         })?;
@@ -238,7 +228,7 @@ impl SDClient for UnixClient {
     }
 
     fn remove_module_value(&self, module_name: &str, path: &str, index: usize) -> Result<RemoveModuleValueResult, SDClientError> {
-        let response: RemoveModuleValueResult = self.process_request(&RemoveModuleValue {
+        let response: RemoveModuleValueResult = process_request(self.get_handle().deref_mut(), &RemoveModuleValue {
             module_name: module_name.to_string(),
             path: path.to_string(),
             index
@@ -248,7 +238,7 @@ impl SDClient for UnixClient {
     }
 
     fn set_module_value(&self, module_name: &str, value: UIPathValue) -> Result<SetModuleValueResult, SDClientError> {
-        let response: SetModuleValueResult = self.process_request(&SetModuleValue {
+        let response: SetModuleValueResult = process_request(self.get_handle().deref_mut(), &SetModuleValue {
             module_name: module_name.to_string(),
             value
         })?;
@@ -257,7 +247,7 @@ impl SDClient for UnixClient {
     }
 
     fn get_stack(&self, serial_number: &str) -> Result<GetStackResult, SDClientError> {
-        let response: GetStackResult = self.process_request(&GetStack {
+        let response: GetStackResult = process_request(self.get_handle().deref_mut(), &GetStack {
             serial_number: serial_number.to_string()
         })?;
 
@@ -265,7 +255,7 @@ impl SDClient for UnixClient {
     }
 
     fn get_stack_names(&self, serial_number: &str) -> Result<GetStackNamesResult, SDClientError> {
-        let response: GetStackNamesResult = self.process_request(&GetStackNames {
+        let response: GetStackNamesResult = process_request(self.get_handle().deref_mut(), &GetStackNames {
             serial_number: serial_number.to_string()
         })?;
 
@@ -273,7 +263,7 @@ impl SDClient for UnixClient {
     }
 
     fn get_current_screen(&self, serial_number: &str) -> Result<GetCurrentScreenResult, SDClientError> {
-        let response: GetCurrentScreenResult = self.process_request(&GetCurrentScreen {
+        let response: GetCurrentScreenResult = process_request(self.get_handle().deref_mut(), &GetCurrentScreen {
             serial_number: serial_number.to_string()
         })?;
 
@@ -281,7 +271,7 @@ impl SDClient for UnixClient {
     }
 
     fn get_button_images(&self, serial_number: &str) -> Result<GetButtonImagesResult, SDClientError> {
-        let response: GetButtonImagesResult = self.process_request(&GetButtonImages {
+        let response: GetButtonImagesResult = process_request(self.get_handle().deref_mut(), &GetButtonImages {
             serial_number: serial_number.to_string()
         })?;
 
@@ -289,7 +279,7 @@ impl SDClient for UnixClient {
     }
 
     fn get_button(&self, serial_number: &str, key: u8) -> Result<GetButtonResult, SDClientError> {
-        let response: GetButtonResult = self.process_request(&GetButton {
+        let response: GetButtonResult = process_request(self.get_handle().deref_mut(), &GetButton {
             serial_number: serial_number.to_string(),
             key
         })?;
@@ -298,7 +288,7 @@ impl SDClient for UnixClient {
     }
 
     fn set_button(&self, serial_number: &str, key: u8, button: Button) -> Result<SetButtonResult, SDClientError> {
-        let response: SetButtonResult = self.process_request(&SetButton {
+        let response: SetButtonResult = process_request(self.get_handle().deref_mut(), &SetButton {
             serial_number: serial_number.to_string(),
             key,
             button
@@ -308,7 +298,31 @@ impl SDClient for UnixClient {
     }
 
     fn clear_button(&self, serial_number: &str, key: u8) -> Result<ClearButtonResult, SDClientError> {
-        let response: ClearButtonResult = self.process_request(&ClearButton {
+        let response: ClearButtonResult = process_request(self.get_handle().deref_mut(), &ClearButton {
+            serial_number: serial_number.to_string(),
+            key
+        })?;
+
+        Ok(response)
+    }
+
+    fn clipboard_status(&self) -> Result<ClipboardStatusResult, SDClientError> {
+        let response: ClipboardStatusResult = process_request_without_data(self.get_handle().deref_mut())?;
+
+        Ok(response)
+    }
+
+    fn copy_button(&self, serial_number: &str, key: u8) -> Result<CopyButtonResult, SDClientError> {
+        let response: CopyButtonResult = process_request(self.get_handle().deref_mut(), &CopyButton {
+            serial_number: serial_number.to_string(),
+            key
+        })?;
+
+        Ok(response)
+    }
+
+    fn paste_button(&self, serial_number: &str, key: u8) -> Result<PasteButtonResult, SDClientError> {
+        let response: PasteButtonResult = process_request(self.get_handle().deref_mut(), &PasteButton {
             serial_number: serial_number.to_string(),
             key
         })?;
@@ -317,7 +331,7 @@ impl SDClient for UnixClient {
     }
 
     fn new_button(&self, serial_number: &str, key: u8) -> Result<NewButtonResult, SDClientError> {
-        let response: NewButtonResult = self.process_request(&NewButton {
+        let response: NewButtonResult = process_request(self.get_handle().deref_mut(), &NewButton {
             serial_number: serial_number.to_string(),
             key
         })?;
@@ -326,7 +340,7 @@ impl SDClient for UnixClient {
     }
 
     fn new_button_from_component(&self, serial_number: &str, key: u8, component_name: &str) -> Result<NewButtonFromComponentResult, SDClientError> {
-        let response: NewButtonFromComponentResult = self.process_request(&NewButtonFromComponent {
+        let response: NewButtonFromComponentResult = process_request(self.get_handle().deref_mut(), &NewButtonFromComponent {
             serial_number: serial_number.to_string(),
             key,
             component_name: component_name.to_string()
@@ -336,7 +350,7 @@ impl SDClient for UnixClient {
     }
 
     fn add_component(&self, serial_number: &str, key: u8, component_name: &str) -> Result<AddComponentResult, SDClientError> {
-        let response: AddComponentResult = self.process_request(&AddComponent {
+        let response: AddComponentResult = process_request(self.get_handle().deref_mut(), &AddComponent {
             serial_number: serial_number.to_string(),
             key,
             component_name: component_name.to_string()
@@ -346,7 +360,7 @@ impl SDClient for UnixClient {
     }
 
     fn get_component_values(&self, serial_number: &str, key: u8, component_name: &str) -> Result<GetComponentValuesResult, SDClientError> {
-        let response: GetComponentValuesResult = self.process_request(&GetComponentValues {
+        let response: GetComponentValuesResult = process_request(self.get_handle().deref_mut(), &GetComponentValues {
             serial_number: serial_number.to_string(),
             key,
             component_name: component_name.to_string()
@@ -356,7 +370,7 @@ impl SDClient for UnixClient {
     }
 
     fn add_component_value(&self, serial_number: &str, key: u8, component_name: &str, path: &str) -> Result<AddComponentValueResult, SDClientError> {
-        let response: AddComponentValueResult = self.process_request(&AddComponentValue {
+        let response: AddComponentValueResult = process_request(self.get_handle().deref_mut(), &AddComponentValue {
             serial_number: serial_number.to_string(),
             key,
             component_name: component_name.to_string(),
@@ -367,7 +381,7 @@ impl SDClient for UnixClient {
     }
 
     fn remove_component_value(&self, serial_number: &str, key: u8, component_name: &str, path: &str, index: usize) -> Result<RemoveComponentValueResult, SDClientError> {
-        let response: RemoveComponentValueResult = self.process_request(&RemoveComponentValue {
+        let response: RemoveComponentValueResult = process_request(self.get_handle().deref_mut(), &RemoveComponentValue {
             serial_number: serial_number.to_string(),
             key,
             component_name: component_name.to_string(),
@@ -379,7 +393,7 @@ impl SDClient for UnixClient {
     }
 
     fn set_component_value(&self, serial_number: &str, key: u8, component_name: &str, value: UIPathValue) -> Result<SetComponentValueResult, SDClientError> {
-        let response: SetComponentValueResult = self.process_request(&SetComponentValue {
+        let response: SetComponentValueResult = process_request(self.get_handle().deref_mut(), &SetComponentValue {
             serial_number: serial_number.to_string(),
             key,
             component_name: component_name.to_string(),
@@ -390,7 +404,7 @@ impl SDClient for UnixClient {
     }
 
     fn remove_component(&self, serial_number: &str, key: u8, component_name: &str) -> Result<RemoveComponentResult, SDClientError> {
-        let response: RemoveComponentResult = self.process_request(&RemoveComponent {
+        let response: RemoveComponentResult = process_request(self.get_handle().deref_mut(), &RemoveComponent {
             serial_number: serial_number.to_string(),
             key,
             component_name: component_name.to_string()
@@ -400,7 +414,7 @@ impl SDClient for UnixClient {
     }
 
     fn push_screen(&self, serial_number: &str, screen: RawButtonPanel) -> Result<PushScreenResult, SDClientError> {
-        let response: PushScreenResult = self.process_request(&PushScreen {
+        let response: PushScreenResult = process_request(self.get_handle().deref_mut(), &PushScreen {
             serial_number: serial_number.to_string(),
             screen
         })?;
@@ -409,7 +423,7 @@ impl SDClient for UnixClient {
     }
 
     fn pop_screen(&self, serial_number: &str) -> Result<PopScreenResult, SDClientError> {
-        let response: PopScreenResult = self.process_request(&PopScreen {
+        let response: PopScreenResult = process_request(self.get_handle().deref_mut(), &PopScreen {
             serial_number: serial_number.to_string()
         })?;
 
@@ -417,7 +431,7 @@ impl SDClient for UnixClient {
     }
 
     fn forcibly_pop_screen(&self, serial_number: &str) -> Result<ForciblyPopScreenResult, SDClientError> {
-        let response: ForciblyPopScreenResult = self.process_request(&ForciblyPopScreen {
+        let response: ForciblyPopScreenResult = process_request(self.get_handle().deref_mut(), &ForciblyPopScreen {
             serial_number: serial_number.to_string()
         })?;
 
@@ -425,7 +439,7 @@ impl SDClient for UnixClient {
     }
 
     fn replace_screen(&self, serial_number: &str, screen: RawButtonPanel) -> Result<ReplaceScreenResult, SDClientError> {
-        let response: ReplaceScreenResult = self.process_request(&ReplaceScreen {
+        let response: ReplaceScreenResult = process_request(self.get_handle().deref_mut(), &ReplaceScreen {
             serial_number: serial_number.to_string(),
             screen
         })?;
@@ -434,7 +448,7 @@ impl SDClient for UnixClient {
     }
 
     fn reset_stack(&self, serial_number: &str, screen: RawButtonPanel) -> Result<ResetStackResult, SDClientError> {
-        let response: ResetStackResult = self.process_request(&ResetStack {
+        let response: ResetStackResult = process_request(self.get_handle().deref_mut(), &ResetStack {
             serial_number: serial_number.to_string(),
             screen
         })?;
@@ -443,7 +457,7 @@ impl SDClient for UnixClient {
     }
 
     fn drop_stack_to_root(&self, serial_number: &str) -> Result<DropStackToRootResult, SDClientError> {
-        let response: DropStackToRootResult = self.process_request(&DropStackToRoot {
+        let response: DropStackToRootResult = process_request(self.get_handle().deref_mut(), &DropStackToRoot {
             serial_number: serial_number.to_string()
         })?;
 
@@ -451,7 +465,7 @@ impl SDClient for UnixClient {
     }
 
     fn commit_changes(&self, serial_number: &str) -> Result<CommitChangesToConfigResult, SDClientError> {
-        let response: CommitChangesToConfigResult = self.process_request(&CommitChangesToConfig {
+        let response: CommitChangesToConfigResult = process_request(self.get_handle().deref_mut(), &CommitChangesToConfig {
             serial_number: serial_number.to_string()
         })?;
 
@@ -459,7 +473,7 @@ impl SDClient for UnixClient {
     }
 
     fn do_button_action(&self, serial_number: &str, key: u8) -> Result<DoButtonActionResult, SDClientError> {
-        let response: DoButtonActionResult = self.process_request(&DoButtonAction {
+        let response: DoButtonActionResult = process_request(self.get_handle().deref_mut(), &DoButtonAction {
             serial_number: serial_number.to_string(),
             key
         })?;
@@ -467,19 +481,48 @@ impl SDClient for UnixClient {
         Ok(response)
     }
 
-    fn send_packet(&self, packet: SocketPacket) -> Result<SocketPacket, SDClientError> {
+    fn send_packet(&self, mut packet: SocketPacket) -> Result<SocketPacket, SDClientError> {
+        let id = rand::thread_rng().sample_iter(&Alphanumeric).take(20).map(char::from).collect::<String>();
+        packet.requester = Some(id.clone());
+
         let mut handle = self.connection.write().unwrap();
-        writeln!(handle.get_mut(), "{}", serde_json::to_string(&packet)?)?;
+        send_packet_as_is(handle.get_mut(), packet)?;
 
-        let mut line = String::new();
-        handle.read_line(&mut line)?;
-
-        Ok(serde_json::from_str(&line)?)
+        read_response(handle.deref_mut(), &id)
     }
 
     fn send_packet_without_response(&self, packet: SocketPacket) -> Result<(), SDClientError> {
         let mut handle = self.connection.write().unwrap();
-        writeln!(handle.get_mut(), "{}", serde_json::to_string(&packet)?)?;
+        send_packet_as_is(handle.get_mut(), packet)?;
         Ok(())
     }
-}
\ No newline at end of file
+}
+
+impl SDSyncEventClient for UnixClient {
+    fn get_event(&self) -> Result<SDGlobalEvent, SDClientError> {
+        loop {
+            let packet = read_socket(self.get_handle().deref_mut())?;
+
+            if packet.ty == "event" {
+                if let Some(data) = packet.data {
+                    return Ok(serde_json::from_value(data)?);
+                }
+            }
+        }
+    }
+}
+
+
+impl SDSyncUpcastRequestClient for UnixClient {
+    fn as_request(self: Arc<Self>) -> Arc<dyn SDSyncRequestClient> {
+        self
+    }
+}
+
+impl SDSyncUpcastEventClient for UnixClient {
+    fn as_event(self: Arc<Self>) -> Arc<dyn SDSyncEventClient> {
+        self
+    }
+}
+
+impl SDSyncClient for UnixClient {}
\ No newline at end of file
